package com.dzeio.openhealth.data

import android.content.Context
import android.util.Log
import androidx.room.AutoMigration
import androidx.room.Database
import androidx.room.Room
import androidx.room.RoomDatabase
import com.dzeio.openhealth.data.food.Food
import com.dzeio.openhealth.data.food.FoodDao
import com.dzeio.openhealth.data.step.Step
import com.dzeio.openhealth.data.step.StepDao
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.data.water.WaterDao
import com.dzeio.openhealth.data.weight.Weight
import com.dzeio.openhealth.data.weight.WeightDao
import com.dzeio.openhealth.utils.ZipFile
import java.io.File
import java.io.IOException
import java.util.zip.ZipInputStream

/**
 * ROOM SQLite database for the application
 *
 * It may be replaced if I want to fully encrypt the database
 */
@Database(
    entities = [
        Weight::class,
        Water::class,
        Step::class,
        Food::class
    ],
    version = 3,
    autoMigrations = [
        AutoMigration(1, 2),
        AutoMigration(2, 3)
    ],
    exportSchema = true
)
abstract class AppDatabase : RoomDatabase() {

    abstract fun weightDao(): WeightDao
    abstract fun waterDao(): WaterDao
    abstract fun stepDao(): StepDao

    abstract fun foodDao(): FoodDao

    companion object {
        /**
         * database name duh
         */
        const val DATABASE_NAME = "open_health"

        // For Singleton instantiation
        @Volatile
        private var instance: AppDatabase? = null

        // get the Database instance
        fun getInstance(context: Context): AppDatabase {
            return instance ?: synchronized(this) {
                instance ?: buildDatabase(context).also { instance = it }
            }
        }

        /**
         * build teh database
         */
        private fun buildDatabase(context: Context): AppDatabase {
            return Room.databaseBuilder(context, AppDatabase::class.java, DATABASE_NAME)
//                .addMigrations(MIGRATION_2_3)
                .build()
        }
    }

    /**
     * export the database to a ZIP file and returns the ByteArray of this ZipFile
     *
     * @param context the application context
     * @return the database as a .zip file
     *
     * based on https://stackoverflow.com/a/73201175/7335674
     */
    fun exportDatabase(context: Context): ByteArray? {
        if (instance == null) return null

        val dbFile = context.getDatabasePath(DATABASE_NAME)
        val dbWalFile = File(dbFile.path + "-wal")
        val dbShmFile = File(dbFile.path + "-shm")
        checkpoint()

        try {
            val output = ZipFile()
            output.addFile(dbFile.name, dbFile)
            if (dbWalFile.exists()) output.addFile(dbWalFile.name, dbWalFile)
            if (dbShmFile.exists()) output.addFile(dbShmFile.name, dbShmFile)
            return output.toByteArray()
        } catch (e: IOException) {
            e.printStackTrace()
            return null
        }
    }

    fun importDatabase(context: Context, zip: ZipInputStream) {
        var entry = zip.nextEntry
        var dbImported = false
        var walImported = false
        var shmImported = false
        val dbFile = context.getDatabasePath(DATABASE_NAME)
        val dbWalFile = File(dbFile.path + "-wal")
        val dbShmFile = File(dbFile.path + "-shm")
        while (entry != null) {
            val direction = File(context.getDatabasePath(DATABASE_NAME).parent, entry.name)
            if (entry.name.equals(dbFile.name)) {
                dbImported = true
                Log.d("AppDatabase", "db imported!")
            } else if (entry.name.equals(dbWalFile.name)) {
                walImported = true
                Log.d("AppDatabase", "wal imported!")
            } else if (entry.name.equals(dbShmFile.name)) {
                shmImported = true
                Log.d("AppDatabase", "shm imported!")
            } else {
                Log.d("AppDatabase", "trying to import random file?!? \"${entry.name}\"")
                continue
            }

            direction.writeBytes(zip.readBytes())
            entry = zip.nextEntry
        }
        if (dbImported) {
            if (!walImported) {
                dbWalFile.delete()
                Log.d("AppDatabase", "deleting wal")
            }
            if (!shmImported) {
                dbShmFile.delete()
                Log.d("AppDatabase", "deleting shm")
            }
        }
    }

    fun checkpoint() {
        val db = this.openHelper.writableDatabase
        db.query("PRAGMA wal_checkpoint(FULL);", emptyArray())
        db.query("PRAGMA wal_checkpoint(TRUNCATE);", emptyArray())
    }
}
