package com.dzeio.openhealth.data.food

import com.dzeio.openhealth.data.openfoodfact.OpenFoodFactService
import com.dzeio.openhealth.utils.NetworkResult
import java.io.IOException
import javax.inject.Inject
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.channelFlow
import kotlinx.coroutines.flow.collectLatest
import kotlinx.coroutines.launch

class FoodRepository @Inject constructor(
    private val dao: FoodDao,
    private val offSource: OpenFoodFactService
) {

    suspend fun searchFood(name: String): Flow<NetworkResult<List<Food>>> = channelFlow {
        val result = NetworkResult<List<Food>>()
        val items = arrayListOf<Food>()
        var otherFinished = false

        launch { // Search OFF
            try {
                val request = offSource.searchProducts(name)
                if (!request.isSuccessful) {
                    if (otherFinished) {
                        result.status = NetworkResult.NetworkStatus.ERRORED
                    } else {
                        otherFinished = true
                    }
                    send(result)
                    return@launch
                }
                val offProducts =
                    offSource.searchProducts(name)
                        .body()?.products?.map { Food.fromOpenFoodFact(it) }

                if (offProducts != null) {
                    items.addAll(offProducts.filterNotNull())
                    result.data = items
                    if (otherFinished) {
                        result.status = NetworkResult.NetworkStatus.FINISHED
                    } else {
                        otherFinished = true
                    }
                    send(result)
                }
            } catch (e: IOException) {
                if (otherFinished) {
                    result.status = NetworkResult.NetworkStatus.ERRORED
                } else {
                    otherFinished = true
                }
                send(result)
            }
        }

        launch { // search local DB
            getAll().collectLatest { list ->
                val filtered = list.filter { it.name.contains(name, true) }
                items.removeAll { it.id > 0 }
                items.addAll(0, filtered)

                result.data = items
                if (otherFinished) {
                    result.status = NetworkResult.NetworkStatus.FINISHED
                } else {
                    otherFinished = true
                }
                send(result)
            }
        }
    }

    fun getAll() = dao.getAll()

    suspend fun add(food: Food) = dao.insert(food)

    fun getById(id: Long) = dao.getOne(id)

    suspend fun delete(food: Food) = dao.delete(food)

    suspend fun update(food: Food) = dao.update(food)
}
