package com.dzeio.openhealth.devices

import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothGatt
import android.bluetooth.BluetoothGattCallback
import android.bluetooth.BluetoothGattCharacteristic
import android.bluetooth.BluetoothGattDescriptor
import android.bluetooth.BluetoothGattService
import android.bluetooth.BluetoothProfile
import android.util.Log
import androidx.annotation.RequiresPermission
import com.dzeio.openhealth.core.Observable
import com.dzeio.openhealth.utils.Bluetooth
import com.dzeio.openhealth.utils.Configuration
import com.dzeio.openhealth.utils.polyfills.writeCharacteristicPoly
import com.dzeio.openhealth.utils.polyfills.writeDescriptorPoly
import java.util.UUID

abstract class BluetoothLeGattDevice(
    var bluetooth: Bluetooth,
    config: Configuration
) : Device<BluetoothDevice>(config) {

    private val status = Observable(ConnectionStatus.DISCONNECTED)

    companion object {

        @RequiresPermission(value = "android.permission.BLUETOOTH_SCAN")
        fun findDevices(
            bluetooth: Bluetooth,
            config: Configuration
        ): Observable<ArrayList<BluetoothLeGattDevice>?> {
            val list = arrayListOf<BluetoothLeGattDevice>()
            val obs = Observable<ArrayList<BluetoothLeGattDevice>?>(null)
            val devices = DeviceFactory.getBluetoothLEDevices(bluetooth, config)

            bluetooth.scanLeDevices {
                if (list.find { item -> item.item?.address == it.address } != null) {
                    return@scanLeDevices false
                }
                for (device in devices) {
                    if (device.isOfType(it)) {
                        device.item = it
                        list.add(device)
                        obs.value = list
                    }
                }
                return@scanLeDevices false
            }
            return obs
        }
    }

    protected val services = arrayListOf<BluetoothGattService>()
    protected lateinit var gatt: BluetoothGatt

    private val gattCallback = ConnectionCallback()

    override fun search(): Observable<BluetoothDevice?> {
        TODO("Not yet implemented")
    }

    override fun connect(): Observable<ConnectionStatus> {
        if (item == null) {
            status.value = ConnectionStatus.ERROR
            return status
        }
        status.value = ConnectionStatus.CONNECTING

        bluetooth.connectGatt(item!!, true, gattCallback)
        return status
    }

    @RequiresPermission(value = "android.permission.BLUETOOTH_CONNECT")
    fun close() {
        gatt.close()
    }

    @RequiresPermission(value = "android.permission.BLUETOOTH_CONNECT")
    protected fun writeBytes(service: UUID, characteristic: UUID, value: ByteArray) {
        val t = findCharacteristic(service, characteristic)
        if (t == null) {
            Log.e("BluetoothLeDevice", "Could not write characteristic")
            return
        }

        gatt.writeCharacteristicPoly(
            t,
            value,
            BluetoothGattCharacteristic.WRITE_TYPE_DEFAULT
        )
    }

    @RequiresPermission(value = "android.permission.BLUETOOTH_CONNECT")
    protected fun setNotification(service: UUID, characteristic: UUID, value: Boolean): Boolean {
        val char = findCharacteristic(service, characteristic)
        if (char == null) {
            Log.e("BluetoothLeDevice", "Could not set notification on characteristic")
            return false
        }
        gatt.setCharacteristicNotification(
            char,
            value
        )
        gatt.writeDescriptorPoly(
            char.getDescriptor(
                BluetoothLeGattUuid.DESCRIPTOR_CLIENT_CHARACTERISTIC_CONFIGURATION
            ),
            BluetoothGattDescriptor.ENABLE_NOTIFICATION_VALUE
        )
        return true
    }

    private fun findCharacteristic(
        service: UUID,
        characteristic: UUID
    ): BluetoothGattCharacteristic? {
        return gatt.services.find { it.uuid == service }?.getCharacteristic(characteristic)
    }

    open fun onPhyUpdate(gatt: BluetoothGatt?, txPhy: Int, rxPhy: Int, status: Int) {}

    open fun onPhyRead(gatt: BluetoothGatt?, txPhy: Int, rxPhy: Int, status: Int) {}

    open fun onConnectionStateChange(gatt: BluetoothGatt?, status: Int, newState: Int) {}

    open fun onServicesDiscovered(gatt: BluetoothGatt?, status: Int) {}

    open fun onCharacteristicRead(
        gatt: BluetoothGatt,
        characteristic: BluetoothGattCharacteristic,
        value: ByteArray,
        status: Int
    ) {
    }

    open fun onCharacteristicWrite(
        gatt: BluetoothGatt?,
        characteristic: BluetoothGattCharacteristic?,
        status: Int
    ) {
    }

    open fun onCharacteristicChanged(
        gatt: BluetoothGatt,
        characteristic: BluetoothGattCharacteristic,
        value: ByteArray
    ) {
    }

    open fun onDescriptorRead(
        gatt: BluetoothGatt,
        descriptor: BluetoothGattDescriptor,
        status: Int,
        value: ByteArray
    ) {
    }

    open fun onDescriptorWrite(
        gatt: BluetoothGatt?,
        descriptor: BluetoothGattDescriptor?,
        status: Int
    ) {
    }

    open fun onReliableWriteCompleted(gatt: BluetoothGatt?, status: Int) {}

    open fun onReadRemoteRssi(gatt: BluetoothGatt?, rssi: Int, status: Int) {}

    open fun onMtuChanged(gatt: BluetoothGatt?, mtu: Int, status: Int) {}

    open fun onServiceChanged(gatt: BluetoothGatt) {}

    private inner class ConnectionCallback : BluetoothGattCallback() {
        override fun onPhyUpdate(
            gatt: BluetoothGatt?,
            txPhy: Int,
            rxPhy: Int,
            status: Int
        ) {
            Log.d("onPhyUpdate", "$gatt, $txPhy, $rxPhy, $status")
            this@BluetoothLeGattDevice.onPhyUpdate(gatt, txPhy, rxPhy, status)
        }

        override fun onPhyRead(
            gatt: BluetoothGatt?,
            txPhy: Int,
            rxPhy: Int,
            status: Int
        ) {
            Log.d("onPhyRead", "$gatt, $txPhy, $rxPhy, $status")
            this@BluetoothLeGattDevice.onPhyRead(gatt, txPhy, rxPhy, status)
        }

        @RequiresPermission("android.permission.BLUETOOTH_CONNECT")
        override fun onConnectionStateChange(
            gatt: BluetoothGatt?,
            status: Int,
            newState: Int
        ) {
            Log.d("onConnectionStateChange", "$gatt, $status, $newState")
            if (gatt != null && newState == BluetoothProfile.STATE_CONNECTED) {
                this@BluetoothLeGattDevice.gatt = gatt
                gatt.discoverServices()
            }
            this@BluetoothLeGattDevice.onConnectionStateChange(gatt, status, newState)
        }

        override fun onServicesDiscovered(gatt: BluetoothGatt?, status: Int) {
            Log.d("onServicesDiscovered", "$gatt, $status")
            if (gatt != null) {
                this@BluetoothLeGattDevice.status.value = ConnectionStatus.CONNECTED
                services.addAll(gatt.services)
            }
            this@BluetoothLeGattDevice.onServicesDiscovered(gatt, status)
        }

        override fun onCharacteristicRead(
            gatt: BluetoothGatt,
            characteristic: BluetoothGattCharacteristic,
            value: ByteArray,
            status: Int
        ) {
            Log.d("onCharacteristicRead", "$gatt, $characteristic, $status, $value")
            this@BluetoothLeGattDevice.onCharacteristicRead(gatt, characteristic, value, status)
        }

        override fun onCharacteristicWrite(
            gatt: BluetoothGatt?,
            characteristic: BluetoothGattCharacteristic?,
            status: Int
        ) {
            Log.d("onCharacteristicWrite", "$gatt, $characteristic, $status")
            this@BluetoothLeGattDevice.onCharacteristicWrite(gatt, characteristic, status)
        }

        override fun onCharacteristicChanged(
            gatt: BluetoothGatt,
            characteristic: BluetoothGattCharacteristic,
            value: ByteArray
        ) {
            Log.d("onCharacteristicChanged", "$gatt, $characteristic, ${byteArrayToHex(value)}")
            this@BluetoothLeGattDevice.onCharacteristicChanged(gatt, characteristic, value)
        }

        override fun onDescriptorRead(
            gatt: BluetoothGatt,
            descriptor: BluetoothGattDescriptor,
            status: Int,
            value: ByteArray
        ) {
            Log.d("onDescriptorRead", "$gatt, $descriptor, $status, $value")
            this@BluetoothLeGattDevice.onDescriptorRead(gatt, descriptor, status, value)
        }

        override fun onDescriptorWrite(
            gatt: BluetoothGatt?,
            descriptor: BluetoothGattDescriptor?,
            status: Int
        ) {
            Log.d("onDescriptorWrite", "$gatt, $descriptor, $status")
            this@BluetoothLeGattDevice.onDescriptorWrite(gatt, descriptor, status)
        }

        override fun onReliableWriteCompleted(gatt: BluetoothGatt?, status: Int) {
            Log.d("onReliableWriteComplete", "$gatt, $status")
            this@BluetoothLeGattDevice.onReliableWriteCompleted(gatt, status)
        }

        override fun onReadRemoteRssi(
            gatt: BluetoothGatt?,
            rssi: Int,
            status: Int
        ) {
            Log.d("onReadRemoteRssi", "$gatt, $rssi, $status")
            this@BluetoothLeGattDevice.onReadRemoteRssi(gatt, rssi, status)
        }

        override fun onMtuChanged(gatt: BluetoothGatt?, mtu: Int, status: Int) {
            Log.d("onMtuChanged", "$gatt, $mtu, $status")
            this@BluetoothLeGattDevice.onMtuChanged(gatt, mtu, status)
        }

        override fun onServiceChanged(gatt: BluetoothGatt) {
            Log.d("onServiceChanged", "$gatt")
            this@BluetoothLeGattDevice.onServiceChanged(gatt)
        }
    }

    private fun byteArrayToHex(arr: ByteArray): String =
        arr.joinToString(" ") { String.format("%02X", it) }
}
