package com.dzeio.openhealth.ui.home

import android.animation.ValueAnimator
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Paint
import android.graphics.RectF
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.navigation.fragment.findNavController
import com.dzeio.charts.Entry
import com.dzeio.charts.axis.Line
import com.dzeio.charts.series.LineSerie
import com.dzeio.openhealth.BuildConfig
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseFragment
import com.dzeio.openhealth.data.water.Water
import com.dzeio.openhealth.databinding.FragmentHomeBinding
import com.dzeio.openhealth.ui.weight.WeightDialog
import com.dzeio.openhealth.utils.ChartUtils
import com.dzeio.openhealth.utils.DrawUtils
import com.google.android.material.color.MaterialColors
import dagger.hilt.android.AndroidEntryPoint
import java.text.DateFormat
import java.util.Date
import java.util.Locale
import kotlin.math.max
import kotlin.math.min

@AndroidEntryPoint
class HomeFragment : BaseFragment<HomeViewModel, FragmentHomeBinding>(HomeViewModel::class.java) {

    override val bindingInflater: (LayoutInflater, ViewGroup?, Boolean) -> FragmentHomeBinding
        get() = FragmentHomeBinding::inflate

    override fun onStart() {
        super.onStart()
        // Bindings
        binding.addWeight.setOnClickListener {
            findNavController().navigate(
                HomeFragmentDirections.actionNavHomeToNavWeightDialog(
                    WeightDialog.DialogTypes.ADD_WEIGHT.ordinal
                )
            )
        }

        /**
         * Water Intake
         */
        binding.fragmentHomeWaterAdd.setOnClickListener {
            val water = viewModel.water.value
            if (water == null || !water.isToday()) {
                val w = Water()
                w.value = viewModel.waterCupSize.value ?: 200
                viewModel.updateWater(w)
            } else {
                water.value += viewModel.waterCupSize.value ?: 200
                viewModel.updateWater(water)
            }
        }

        viewModel.waterUnit.observe(viewLifecycleOwner) {
            updateWaterTotal()
        }

        viewModel.dailyWaterIntake.observe(viewLifecycleOwner) {
            updateWaterTotal()
        }

        binding.fragmentHomeWaterRemove.setOnClickListener {
            val water = viewModel.water.value
            if (water != null) {
                water.value -= viewModel.waterCupSize.value ?: 200
                if (water.value <= 0) {
                    viewModel.deleteWater(water)
                } else {
                    viewModel.updateWater(water)
                }
            }
        }

        // handle button to go to weight home
        binding.listWeight.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavListWeight())
        }

        // handle button to go to water intake home
        binding.gotoWaterHome.setOnClickListener {
            findNavController().navigate(HomeFragmentDirections.actionNavHomeToNavWaterHome())
        }

        binding.weightGraph.apply {
            LineSerie(this)
            animator.enabled = false
            xAxis.apply {
                dataWidth = 2.4192e+9
                scrollEnabled = true
                onValueFormat = onValueFormat@{
                    val formatter = DateFormat.getDateTimeInstance(
                        DateFormat.SHORT,
                        DateFormat.SHORT,
                        Locale.getDefault()
                    )
                    return@onValueFormat formatter.format(Date(it.toLong()))
                }
            }
            ChartUtils.materielTheme(this, requireView())
        }

        // update the graph when the weight unit change
        viewModel.massUnit.observe(viewLifecycleOwner) { unit ->
            binding.weightGraph.yAxis.onValueFormat = { getString(unit.unit, unit.fromKilogram(it)) }
            binding.weightGraph.refresh()
        }

        if (BuildConfig.DEBUG) {
            binding.gotoTests.apply {
                visibility = View.VISIBLE
                setOnClickListener {
                    findNavController().navigate(
                        HomeFragmentDirections.actionNavHomeToTestsFragment()
                    )
                }
            }
        }

        // Update the water intake Graph when the water intake changes
        viewModel.water.observe(viewLifecycleOwner) {
            updateWaterGraph()
        }

        // Update the steps Graph when the steps count changes
        viewModel.steps.observe(viewLifecycleOwner) {
            binding.stepsCurrent.text = getString(
                R.string.steps_count,
                it
            )
        }

        // Update the steps Graph when the goal changes
        viewModel.stepsGoal.observe(viewLifecycleOwner) {
            if (it == null) {
                binding.stepsTotal.visibility = View.GONE
                return@observe
            }
            binding.stepsTotal.visibility = View.VISIBLE
            binding.stepsTotal.text = getString(
                R.string.steps_count,
                it
            )
        }

        // update the graph when the weight changes
        viewModel.weights.observe(viewLifecycleOwner) {
            updateWeightGraph()
        }

        // update the graph when the goal weight change
        viewModel.goalWeight.observe(viewLifecycleOwner) {
            updateWeightGraph()
        }
    }

    /**
     * Function that update the graph for the weight
     */
    private fun updateWeightGraph() {
        val values = viewModel.weights.value ?: arrayListOf()
        val goal = viewModel.goalWeight.value

        val chart = binding.weightGraph
        val serie = chart.series[0] as LineSerie

        val entries: ArrayList<Entry> = values.map {
            Entry(
                it.timestamp.toDouble(),
                it.weight
            )
        } as ArrayList<Entry>

        serie.entries = entries

        chart.yAxis.clearLines()
        if (goal != null) {
            chart.yAxis.addLine(
                goal,
                Line(true, Paint(chart.yAxis.linePaint).apply { strokeWidth = 4f })
            )
        }

        if (entries.isEmpty()) {
            chart.xAxis.x = 0.0
        } else {
            chart.xAxis.x = entries.last().x - chart.xAxis.dataWidth!!
        }

        chart.refresh()
    }

    /**
     * the waterintake old value to keep for value update
     */
    private var oldValue = 0

    /**
     * function that update the water count in the home page
     */
    private fun updateWaterGraph() {
        val newValue = viewModel.water.value?.value ?: 0
        val daily = viewModel.dailyWaterIntake.value
        val unit = viewModel.waterUnit.value

        if (unit == null) {
            return
        }

        // Update the count
        binding.fragmentHomeWaterCurrent.text =
            getString(
                unit.unit,
                (newValue * unit.modifier).toInt()
            )

        if (daily == null) {
            return
        }

        // get the with/height of the ImageView
        var width = 1500
        var height = 750

        if (binding.background.width != 0) {
            width = binding.background.width
            height = binding.background.height
        }

        // Prepare the update animation
        val animator = ValueAnimator.ofInt(
            this.oldValue,
            newValue
        )
        animator.duration = 300 // ms
        animator.addUpdateListener {
            val localView = view
            if (localView == null) {
                return@addUpdateListener
            }
            this.oldValue = (it.animatedValue as Int)
            val value = 100 * it.animatedValue as Int / daily.toFloat()

            val graph = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888)
            val canvas = Canvas(graph)
            val rect = RectF(
                10f,
                15f,
                90f,
                85f
            )

            // background Arc
            DrawUtils.drawArc(
                canvas,
                100f,
                rect,
                MaterialColors.getColor(
                    localView,
                    com.google.android.material.R.attr.colorOnPrimary
                ),
                3f
            )

            // Draw the big Arc
            DrawUtils.drawArc(
                canvas,
                min(max(value, 0.01f), 100f),
                rect,
                MaterialColors.getColor(
                    localView,
                    com.google.android.material.R.attr.colorPrimary
                ),
                6f
            )

            // save the canvas
            canvas.save()

            // send it
            binding.background.setImageBitmap(graph)
        }

        // start the animation
        animator.start()
    }

    private fun updateWaterTotal() {
        val unit = viewModel.waterUnit.value
        val value = viewModel.dailyWaterIntake.value
        if (unit == null) {
            return
        }

        if (value == null) {
            binding.fragmentHomeWaterTotal.visibility = View.GONE
            return
        }
        binding.fragmentHomeWaterTotal.visibility = View.VISIBLE

        binding.fragmentHomeWaterTotal.text = getString(
            unit.unit,
            unit.fromMilliliter(value)
        )
    }
}
