package com.dzeio.openhealth.ui.settings

import android.content.SharedPreferences
import android.os.Build
import android.os.Bundle
import android.text.InputType
import android.util.Log
import androidx.preference.EditTextPreference
import androidx.preference.ListPreference
import androidx.preference.PreferenceFragmentCompat
import androidx.preference.PreferenceManager
import com.dzeio.openhealth.Application
import com.dzeio.openhealth.BuildConfig
import com.dzeio.openhealth.R
import com.dzeio.openhealth.Settings
import com.dzeio.openhealth.units.Mass
import com.dzeio.openhealth.units.Volume
import com.dzeio.openhealth.utils.Configuration
import com.dzeio.openhealth.utils.LocaleUtils
import com.dzeio.openhealth.utils.fields.IntEditTextPreference
import dagger.hilt.android.AndroidEntryPoint
import java.util.Locale
import javax.inject.Inject

@AndroidEntryPoint
class SettingsFragment : PreferenceFragmentCompat() {

    @Inject
    lateinit var config: Configuration

    private companion object {
        const val TAG = "${Application.TAG}/Settings"
    }

    val settings: SharedPreferences by lazy {
        PreferenceManager.getDefaultSharedPreferences(requireContext())
    }

    override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
        setPreferencesFromResource(R.xml.preferences, rootKey)

        // Force only numbers on Goal
        val weightGoal = findPreference<EditTextPreference>("tmp_goal_weight")
        weightGoal?.apply {
            setOnBindEditTextListener {
                it.setSelectAllOnFocus(true)
                it.inputType = InputType.TYPE_CLASS_NUMBER or InputType.TYPE_NUMBER_FLAG_DECIMAL
            }

            val value = config.getFloat(Settings.WEIGHT_GOAL)
            setOnPreferenceClickListener {
                if (value.value != null) {
                    val unit = config.getInt(Settings.MASS_UNIT).value
                    text = if (unit == null) {
                        value.value!!.toString()
                    } else {
                        val modifier = Mass.find(unit).modifier
                        (value.value!! * modifier).toString()
                    }
                }
                return@setOnPreferenceClickListener true
            }

            setOnPreferenceChangeListener { _, newValue ->
                val unit = config.getInt(Settings.MASS_UNIT).value
                var modifier = Mass.KILOGRAM.modifier
                if (unit != null) {
                    modifier = Mass.find(unit).modifier
                }

                value.value = ((newValue as String).toFloat() / modifier)

                return@setOnPreferenceChangeListener false
            }
        }

        val height = findPreference<EditTextPreference>("tmp_height")
        height?.apply {
            setOnBindEditTextListener {
                it.setSelectAllOnFocus(true)
                it.inputType = InputType.TYPE_CLASS_NUMBER or InputType.TYPE_NUMBER_FLAG_DECIMAL
            }

            val value = config.getInt(Settings.USER_HEIGHT)

            setOnPreferenceClickListener {
                text = value.value?.toString()
                return@setOnPreferenceClickListener true
            }

            setOnPreferenceChangeListener { _, newValue ->

                value.value = (newValue as String).toInt()

                return@setOnPreferenceChangeListener false
            }
        }

        val activityPreference = findPreference<ListPreference>("tmp." + Settings.USER_ACTIVITY_LEVEL)
        activityPreference?.apply {
            val value = config.getInt(Settings.USER_ACTIVITY_LEVEL)
            setOnPreferenceClickListener {
                if (value.value != null) {
                    setValueIndex(value.value!!)
                }
                return@setOnPreferenceClickListener true
            }
            setOnPreferenceChangeListener { _, newValue ->
                value.value = findIndexOfValue(newValue.toString())
                return@setOnPreferenceChangeListener false
            }
        }

        val biologicalSexPreference = findPreference<ListPreference>("tmp." + Settings.USER_BIOLOGICAL_SEX)
        biologicalSexPreference?.apply {
            val value = config.getInt(Settings.USER_BIOLOGICAL_SEX)
            setOnPreferenceClickListener {
                if (value.value != null) {
                    setValueIndex(value.value!!)
                }
                return@setOnPreferenceClickListener true
            }
            setOnPreferenceChangeListener { _, newValue ->
                value.value = findIndexOfValue(newValue.toString())
                return@setOnPreferenceChangeListener false
            }
        }

        val languagesPreference = findPreference<ListPreference>(Settings.APP_LANGUAGE)
        Log.d(TAG, Locale.getDefault().language)
        languagesPreference?.apply {
            entries = BuildConfig.LOCALES
            entryValues = BuildConfig.LOCALES
            setDefaultValue(Locale.getDefault().language)
        }

        findPreference<ListPreference>("tmp." + Settings.VOLUME_UNIT)?.apply {
            entries = Volume.values().map { getString(it.singular) }.toTypedArray()
            entryValues = Volume.values().map { it.ordinal.toString() }.toTypedArray()
            val unit = config.getInt(Settings.VOLUME_UNIT)

            setOnPreferenceClickListener {
                if (unit.value != null) setValueIndex(unit.value!!)
                return@setOnPreferenceClickListener true
            }

            setOnPreferenceChangeListener { _, newValue ->
                val nv = (newValue as String).toIntOrNull()
                unit.value = nv
                return@setOnPreferenceChangeListener false
            }
        }

        findPreference<ListPreference>("tmp." + Settings.MASS_UNIT)?.apply {
            entries = Mass.values().map { getString(it.singular) }.toTypedArray()
            entryValues = Mass.values().map { it.ordinal.toString() }.toTypedArray()
            val unit = config.getInt(Settings.MASS_UNIT)

            setOnPreferenceClickListener {
                if (unit.value != null) setValueIndex(unit.value!!)
                return@setOnPreferenceClickListener true
            }

            setOnPreferenceChangeListener { _, newValue ->
                val nv = (newValue as String).toIntOrNull()
                unit.value = nv
                return@setOnPreferenceChangeListener false
            }
        }

        findPreference<IntEditTextPreference>("tmp." + Settings.WATER_INTAKE_DAILY_GOAL)?.apply {
            val value = config.getInt(Settings.WATER_INTAKE_DAILY_GOAL)
            val unit = config.getInt(Settings.VOLUME_UNIT)

            setOnPreferenceClickListener {
                if (value.value == null) {
                    return@setOnPreferenceClickListener true
                }
                text = if (unit.value == null) {
                    value.value!!.toString()
                } else {
                    Volume.find(unit.value!!).fromMilliliter(value.value!!).toString()
                }
                return@setOnPreferenceClickListener true
            }

            setOnPreferenceChangeListener { _, newValue ->
                val nv = (newValue as String).toIntOrNull()

                value.value = if (unit.value == null || nv == null) {
                    nv
                } else {
                    Volume.find(unit.value!!).toMilliliter(nv)
                }

                return@setOnPreferenceChangeListener false
            }
        }

        // Update App Locale
        languagesPreference?.setOnPreferenceChangeListener { _, newValue ->
            LocaleUtils.setLanguage(requireContext(), newValue as String)
            if (Build.VERSION.SDK_INT < Build.VERSION_CODES.TIRAMISU) {
                requireActivity().recreate()
            }

            return@setOnPreferenceChangeListener true
        }
    }
}
