package com.dzeio.openhealth.ui.weight

import android.graphics.drawable.Drawable
import android.util.Log
import android.view.LayoutInflater
import androidx.lifecycle.lifecycleScope
import androidx.navigation.fragment.navArgs
import com.dzeio.openhealth.R
import com.dzeio.openhealth.core.BaseDialog
import com.dzeio.openhealth.databinding.DialogWeightBinding
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import dagger.hilt.android.AndroidEntryPoint

@AndroidEntryPoint
class WeightDialog :
    BaseDialog<WeightDialogViewModel, DialogWeightBinding>(WeightDialogViewModel::class.java) {

    private val args: WeightDialogArgs by navArgs()

    override val bindingInflater: (LayoutInflater) -> DialogWeightBinding =
        DialogWeightBinding::inflate

    override fun onBuilderInit(builder: MaterialAlertDialogBuilder) {
        super.onBuilderInit(builder)

        val dialogType = DialogTypes.values()[args.dialogType]

        builder.apply {
            setTitle(dialogType.title)
            setIcon(dialogType.icon)
            setPositiveButton(R.string.validate) { dialog, _ ->
                runAction(0)
                dialog.dismiss()
            }
            setNegativeButton(R.string.cancel) { dialog, _ ->
                runAction(1)
                dialog.cancel()
            }
            if (dialogType.thirdText != null) {
                setNeutralButton(dialogType.thirdText) { dialog, _ ->
                    runAction(2)
                    dialog.dismiss()
                }
            }
        }
    }

    override fun onCreated() {
        super.onCreated()
        setValue(0f)

        if (args.dialogType == DialogTypes.ADD_WEIGHT.ordinal) {
            viewModel.initWithWeight(null)
            viewModel.weight.observe(this) {
                if (it != null) setValue(it.weight * viewModel.format.modifier)
            }
        } else if (args.dialogType == DialogTypes.EDIT_GOAL.ordinal) {
            viewModel.goalWeight.observe(this) {
                if (it != null) setValue(it * viewModel.format.modifier)
            }
        }

        // init form
        binding.kg.maxValue = 999
        binding.kg.minValue = 0

        binding.gram.maxValue = 9
        binding.gram.minValue = 0
    }

    private fun setValue(value: Float) {
        Log.d("TAG", "Setting dialog value to $value")
        val kg = value.toInt()
        val g = (value - kg) * 10

        binding.kg.value = kg
        binding.gram.value = g.toInt()
    }

    private fun getValue(): Float {
        var finalValue: Float = binding.kg.value.toFloat()
        finalValue += binding.gram.value.toFloat() / 10
        return finalValue
    }

    /**
     * @param click 0 = validate, 1 = cancel, 2 = third option
     */
    private fun runAction(click: Int) {
        when (DialogTypes.values()[args.dialogType]) {
            DialogTypes.ADD_WEIGHT -> {
                val value = getValue()
                if (click == 0) {
                    lifecycleScope.launchWhenCreated {
                        viewModel.addWeight(value)
                    }
                }
            }
            DialogTypes.EDIT_GOAL -> {
                if (click == 1) {
                    return
                }

                if (click == 2) {
                    viewModel.setWeightGoal(null)
                    return
                }

                viewModel.setWeightGoal(getValue())
                return
            }
        }
    }

    enum class DialogTypes(
        val title: String,
        val icon: Drawable? = null,
        val thirdText: Int? = null
    ) {
        ADD_WEIGHT("Add Weight", null),
        EDIT_GOAL("Edit Goal", null, R.string.goal_remove)
    }
}
