package com.dzeio.openhealth.utils

import android.annotation.SuppressLint
import android.bluetooth.BluetoothAdapter
import android.bluetooth.BluetoothDevice
import android.bluetooth.BluetoothGattCallback
import android.bluetooth.BluetoothManager
import android.bluetooth.le.ScanCallback
import android.bluetooth.le.ScanResult
import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.PackageManager
import android.util.Log
import android.widget.Toast
import androidx.annotation.RequiresPermission
import com.dzeio.openhealth.utils.polyfills.getBluetoothDevice

class Bluetooth(
    private val context: Context
) {
    companion object {
        const val TAG = "Bluetooth"
    }

    private lateinit var adapter: BluetoothAdapter

    init {
        if (!context.packageManager.hasSystemFeature(PackageManager.FEATURE_BLUETOOTH)) {
            throw Exception("Phone missing the Bluetooth feature")
        }
        try {
            adapter = (context.getSystemService(Context.BLUETOOTH_SERVICE) as BluetoothManager).adapter
        } catch (e: Exception) {
            Log.e(TAG, e.message ?: "error getting default adapter")
        }
        if (!adapter.isEnabled) {
            Toast.makeText(context, "Bluetooth is not enabled", Toast.LENGTH_LONG).show()
            Log.e(TAG, "Bluetooth is not enabled")
        }
    }

    @SuppressLint("HardwareIds")
    fun getAddress(): String {
        return adapter.address
    }

    @RequiresPermission(value = "android.permission.BLUETOOTH_CONNECT")
    fun getName(): String {
        return adapter.name
    }

    /**
     * Make the current phone discoverable through bluetooth scan
     */
//    fun makeDeviceDiscoverable(time: Int = 600) {
//        val intent = Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE).apply {
//            putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, time)
//        }
//        activity.startActivityForResult(intent, 1)
//    }

    fun getDiscoverableIntent(time: Int = 600): Intent {
        return Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE).apply {
            putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, time)
        }
    }

    private lateinit var cb: (device: BluetoothDevice) -> Boolean

    private val receiver: BroadcastReceiver = object : BroadcastReceiver() {
        @RequiresPermission(allOf = ["android.permission.BLUETOOTH_SCAN", "android.permission.BLUETOOTH_CONNECT"])
        override fun onReceive(context: Context, intent: Intent) {
            val action = intent.action
            if (BluetoothDevice.ACTION_FOUND == action) {
                val device =
                    intent.getBluetoothDevice()
                // val rssi = intent.getShortExtra(BluetoothDevice.EXTRA_RSSI, Short.MIN_VALUE).toInt()

                if (device != null) {
                    Log.d(TAG, "Device found! (${device.name ?: device.address})")
                    val end = cb(device)
                    if (end) {
                        stopScan()
                    }
                }
            }
        }
    }

    /**
     * Callback return boolean if true it stop scan
     */
    @RequiresPermission(value = "android.permission.BLUETOOTH_SCAN")
    fun scanDevices(callback: (device: BluetoothDevice) -> Boolean): Boolean {
        val filter = IntentFilter(BluetoothDevice.ACTION_FOUND)
        context.registerReceiver(receiver, filter)
        cb = callback
        return adapter.startDiscovery()
    }

    private var leCallback: ((device: BluetoothDevice) -> Boolean)? = null
    private val scanCallback = object : ScanCallback() {
        @RequiresPermission(allOf = ["android.permission.BLUETOOTH_SCAN", "android.permission.BLUETOOTH_CONNECT"])
        override fun onScanResult(callbackType: Int, result: ScanResult?) {
            super.onScanResult(callbackType, result)
            if (result != null) {
                val device = result.device
                Log.d(TAG, "Device found! (${device.name ?: device.address})")
                val doStop = leCallback?.invoke(device)
                if (doStop == true) {
                    stopScan()
                }
            }
        }
    }

    /**
     * Callback return boolean if true it stop scan
     */
    @RequiresPermission(value = "android.permission.BLUETOOTH_SCAN")
    fun scanLeDevices(callback: (device: BluetoothDevice) -> Boolean) {
        val pouet = adapter.bluetoothLeScanner
        leCallback = callback
        pouet.startScan(scanCallback)
    }

    @RequiresPermission(value = "android.permission.BLUETOOTH_SCAN")
    fun stopScan() {
        adapter.bluetoothLeScanner.stopScan(scanCallback)
        adapter.cancelDiscovery()
        try {
            context.unregisterReceiver(receiver)
        } catch (it: IllegalArgumentException) {
            Log.i(TAG, "Seems like it was already unloaded", it)
        }
    }

    @SuppressLint("MissingPermission")
    fun connectGatt(
        device: BluetoothDevice,
        autoConnect: Boolean,
        callback: BluetoothGattCallback
    ) {
        device.connectGatt(context, autoConnect, callback)
    }
}
